/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 */
package jspritemaker.view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import jspritemaker.model.Sprite;

/**
 * JSpriteMaker 68k
 * Version 1.0
 * Copyright (C) 2002 John David Ratliff
 */
public class SpriteDialog implements ActionListener, FocusListener, KeyListener {
	private JDialog dialog;
	private Sprite model;
	private int type;
	
	private JTextField jtfWidth, jtfHeight, jtfCalc, jtfLight, jtfDark, jtfMask;
	private JRadioButton jrb89, jrb92;
	private JCheckBox jcbDefault;
	
	private int width, height, calc, value;
	
	// class constants
	public final static int NEW_SPRITE_DIALOG = 0;
	public final static int RESIZER_DIALOG    = 1;
	public final static int PROPERTIES_DIALOG = 2;
	
	public final static String[] TITLES = {"New Sprite Creation","Resize Sprite","Sprite Properties"};
	
	/* constructors */
	
	public SpriteDialog(JFrame frame, Sprite model, int type) {
		this.model = model;
		this.type = type;
		this.value = JOptionPane.CANCEL_OPTION;
		
		// initialize the dialog
		this.initDialog(frame);
	}
	
	/* private methods */
	
	private void initDialog(JFrame frame) {
		this.dialog = new JDialog(frame,TITLES[type],true);
		
		// add event listeners
		dialog.addKeyListener(this);
		
		dialog.setSize(getPreferredSize());
		
		// WARNING: compile this code with Java 2 SDK 1.3, not 1.4
		// method was changed in 1.4, and is NOT backward compatible
		dialog.setLocationRelativeTo(frame);
		
		// replace the content pane
		dialog.setContentPane(buildContentPane());
	}
	
	private JPanel buildSpriteDimensionsPanel(int width, int height, int calc) {
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		JPanel panel = new JPanel(layout);
		JPanel radioPanel = new JPanel();
		JLabel jlWidth, jlHeight, jlCalc;
		ButtonGroup radio;
		
		// create the JTextFields
		jtfWidth = new JTextField("" + width,7);
		jtfWidth.addFocusListener(this);
		
		jtfHeight = new JTextField("" + height,7);
		jtfHeight.addFocusListener(this);
		
		// create the JLabels
		jlWidth = new JLabel("Sprite Width:");
		jlWidth.setDisplayedMnemonic(KeyEvent.VK_W);
		jlWidth.setLabelFor(jtfWidth);
		
		jlHeight = new JLabel("Sprite Height:");
		jlHeight.setDisplayedMnemonic(KeyEvent.VK_H);
		jlHeight.setLabelFor(jtfHeight);
		
		jlCalc = new JLabel("Calculator");
		
		// create the JRadioButtons
		jrb89 = new JRadioButton("TI-89",((calc == Sprite.CALC_TI89) ? true : false));
		jrb89.setMnemonic(KeyEvent.VK_8);
		
		jrb92 = new JRadioButton("TI-92+",((calc == Sprite.CALC_TI92) ? true : false));
		jrb92.setMnemonic(KeyEvent.VK_9);
		
		// create the ButtonGroup
		radio = new ButtonGroup();
		radio.add(jrb89);
		radio.add(jrb92);
		
		// add components to the radio panel
		radioPanel.add(jrb89);
		radioPanel.add(jrb92);
		
		// setup the GridBagConstraints
		gbc.fill = GridBagConstraints.NONE;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		
		// set the GridBagLayout properties for each component and attach
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 0.0;
		gbc.insets.left = 3;
		gbc.insets.right = 10;
		layout.setConstraints(jlWidth,gbc);
		panel.add(jlWidth);
		
		gbc.gridx = 1;
		gbc.weightx = 1.0;
		gbc.insets.left = 0;
		gbc.insets.right = 0;
		layout.setConstraints(jtfWidth,gbc);
		panel.add(jtfWidth);
		
		gbc.gridy = 1;
		gbc.gridx = 0;
		gbc.weightx = 0.0;
		gbc.insets.left = 3;
		gbc.insets.right = 10;
		layout.setConstraints(jlHeight,gbc);
		panel.add(jlHeight);
		
		gbc.gridx = 1;
		gbc.weightx = 1.0;
		gbc.insets.left = 0;
		gbc.insets.right = 0;
		layout.setConstraints(jtfHeight,gbc);
		panel.add(jtfHeight);
		
		gbc.gridx = 2;
		gbc.gridy = 0;
		gbc.weightx = 0.0;
		gbc.anchor = GridBagConstraints.CENTER;
		layout.setConstraints(jlCalc,gbc);
		panel.add(jlCalc);
		
		gbc.gridy = 1;
		gbc.weightx = 1.0;
		layout.setConstraints(radioPanel,gbc);
		panel.add(radioPanel);
		
		return panel;
	}
	
	private JPanel buildSpriteNamesPanel(String light, String dark, String mask, boolean useDefaultNames) {
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		JPanel panel = new JPanel(layout);
		JLabel jlLight, jlDark, jlMask;
		
		// create the JCheckBoxes
		jcbDefault = new JCheckBox("Use Default Sprite Names",useDefaultNames);
		jcbDefault.setMnemonic(KeyEvent.VK_U);
		jcbDefault.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				jtfLight.setEnabled(!jtfLight.isEnabled());
				jtfDark.setEnabled(!jtfDark.isEnabled());
				jtfMask.setEnabled(!jtfMask.isEnabled());
			}
		});
		
		// create the JTextFields
		jtfLight = new JTextField(light,14);
		jtfLight.addFocusListener(this);
		
		jtfDark = new JTextField(dark,14);
		jtfDark.addFocusListener(this);
		
		jtfMask = new JTextField(mask,14);
		jtfMask.addFocusListener(this);
		
		if (useDefaultNames) {
			jtfLight.setEnabled(false);
			jtfDark.setEnabled(false);
			jtfMask.setEnabled(false);
		}
		
		// create the JLabels
		jlLight = new JLabel("Light Plane Name:");
		jlLight.setDisplayedMnemonic(KeyEvent.VK_L);
		jlLight.setLabelFor(jtfLight);
		
		jlDark = new JLabel("Dark Plane Name:");
		jlDark.setDisplayedMnemonic(KeyEvent.VK_D);
		jlDark.setLabelFor(jtfDark);
		
		jlMask = new JLabel("Mask Plane Name:");
		jlMask.setDisplayedMnemonic(KeyEvent.VK_M);
		jlMask.setLabelFor(jtfMask);
		
		// set the GridBagLayout properties for each component and attach
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 2;
		gbc.anchor = GridBagConstraints.CENTER;
		gbc.weightx = 1.0;
		layout.setConstraints(jcbDefault,gbc);
		panel.add(jcbDefault);
		
		gbc.gridy = 1;
		gbc.gridwidth = 1;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.insets.left = 3;
		gbc.insets.right = 10;
		gbc.weightx = 0.0;
		layout.setConstraints(jlLight,gbc);
		panel.add(jlLight);
		
		gbc.gridx = 1;
		gbc.insets.left = 0;
		gbc.insets.right = 0;
		gbc.weightx = 1.0;
		layout.setConstraints(jtfLight,gbc);
		panel.add(jtfLight);
		
		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.insets.left = 3;
		gbc.insets.right = 10;
		gbc.weightx = 0.0;
		layout.setConstraints(jlDark,gbc);
		panel.add(jlDark);
		
		gbc.gridx = 1;
		gbc.insets.left = 0;
		gbc.insets.right = 0;
		gbc.weightx = 1.0;
		layout.setConstraints(jtfDark,gbc);
		panel.add(jtfDark);
		
		gbc.gridx = 0;
		gbc.gridy = 3;
		gbc.insets.left = 3;
		gbc.insets.right = 10;
		gbc.weightx = 0.0;
		layout.setConstraints(jlMask,gbc);
		panel.add(jlMask);
		
		gbc.gridx = 1;
		gbc.insets.left = 0;
		gbc.insets.right = 0;
		gbc.weightx = 1.0;
		layout.setConstraints(jtfMask,gbc);
		panel.add(jtfMask);

		return panel;
	}
	
	private JPanel buildButtonPanel() {
		JPanel panel = new JPanel();
		JButton ok, cancel;
		
		// create the JButtons
		ok = new JButton("OK");
		ok.setMnemonic(KeyEvent.VK_O);
		ok.setActionCommand("okay");
		ok.addActionListener(this);
		
		cancel = new JButton("Cancel");
		cancel.setMnemonic(KeyEvent.VK_C);
		cancel.setActionCommand("cancel");
		cancel.addActionListener(this);
		
		// add the components to the panel
		panel.add(ok);
		panel.add(cancel);
		
		return panel;
	}
	
	// sprite panel creation
	private JPanel buildFullPane(int width, int height, int calc, String light, String dark, String mask, boolean useDefaultNames) {
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		JPanel panel = new JPanel(layout);
		JPanel panel1, panel2, panel3;
		
		// set layout for top controls (sprite dimensions)
		panel1 = this.buildSpriteDimensionsPanel(width,height,calc);
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		layout.setConstraints(panel1,gbc);
		panel.add(panel1);
		
		// set layout for bottom controls (sprite variable names)
		panel2 = this.buildSpriteNamesPanel(light,dark,mask,useDefaultNames);
		gbc.gridy = 1;
		gbc.gridx = 0;
		layout.setConstraints(panel2,gbc);
		panel.add(panel2);
		
		// set layout for dialog controls
		panel3 = this.buildButtonPanel();
		gbc.gridy = 2;
		gbc.gridx = 0;
		gbc.anchor = GridBagConstraints.SOUTH;
		layout.setConstraints(panel3,gbc);
		panel.add(panel3);
		
		return panel;
	}
	
	private JPanel buildResizerPane(int width, int height, int calc) {
		JPanel panel = new JPanel(new BorderLayout());
		
		panel.add(buildSpriteDimensionsPanel(width, height, calc),BorderLayout.NORTH);
		panel.add(buildButtonPanel(),BorderLayout.CENTER);
		
		return panel;
	}
	
	private JPanel buildContentPane() {
		JPanel panel;
		
		if (type == RESIZER_DIALOG) {
			panel = buildResizerPane(model.getWidth(),model.getHeight(),model.getCalc());
		} else if (type == NEW_SPRITE_DIALOG) {
			panel = buildFullPane(16,16,Sprite.CALC_TI89,"light","dark","mask",true);
		} else {
			panel = buildFullPane(model.getWidth(),model.getHeight(),
					      model.getCalc(),model.getLightPlaneName(),
					      model.getDarkPlaneName(),
					      model.getMaskName(),false);
		}
		
		return panel;
	}
	
	private int getMaxWidth() {
		return ((calc == Sprite.CALC_TI89) ? 160 : 240);
	}
	
	private int getMaxHeight() {
		return ((calc == Sprite.CALC_TI89) ? 100: 128);
	}
	
	private boolean isValidWidth() {
		return ((width > 0) && (width <= getMaxWidth()));
	}
	
	private boolean isValidHeight() {
		return ((height > 0) && (height <= getMaxHeight()));
	}
	
	private boolean checkValidInput() {
		boolean valid = false;
		String INVALID_DIMENSION_ERROR = "Error: Invalid Sprite Dimension";
		
		if (jrb89.isSelected()) {
			calc = Sprite.CALC_TI89;
		} else {
			calc = Sprite.CALC_TI92;
		}
		
		try {
			width = Integer.parseInt(jtfWidth.getText());
			height = Integer.parseInt(jtfHeight.getText());
			
			if ((width % 8) != 0) {
				// width must be a multiple of 8
				JOptionPane.showMessageDialog(dialog,"Sprite width must be a multiple of 8 (i.e. 8, 16, 24, etc)",
					INVALID_DIMENSION_ERROR,JOptionPane.ERROR_MESSAGE);
				jtfWidth.requestFocus();
			} else if (!isValidWidth()) {
				JOptionPane.showMessageDialog(dialog,"Width must be between 8 and " +
					this.getMaxWidth() + " pixels.",INVALID_DIMENSION_ERROR,
					JOptionPane.ERROR_MESSAGE);
				jtfWidth.requestFocus();
			} else if (!isValidHeight()) {
				JOptionPane.showMessageDialog(dialog,"Height must be between 1 and " +
					this.getMaxHeight() + " pixels.",INVALID_DIMENSION_ERROR,
					JOptionPane.ERROR_MESSAGE);
				jtfHeight.requestFocus();
			} else {
				valid = true;
			}
		} catch (NumberFormatException e) {
			JOptionPane.showMessageDialog(dialog,"Sprite Dimensions must be integers.",
				INVALID_DIMENSION_ERROR,JOptionPane.ERROR_MESSAGE);
		}
		
		return valid;
	}
	
	private void ok() {
		if (checkValidInput()) {
			value = JOptionPane.OK_OPTION;
			dialog.setVisible(false);
			dialog.dispose();
		}
	}
	
	private void cancel() {
		this.dialog.setVisible(false);
		this.dialog.dispose();
	}
	
	/* public methods */
	
	public Dimension getPreferredSize() {
		if (type != RESIZER_DIALOG) {
			return new Dimension(400,300);
		}
		
		return new Dimension(400,125);
	}
	
	public int isValid() {
		this.dialog.setVisible(true);
		
		return this.value;
	}
	
	public int getWidth() {
		return this.width;
	}
	
	public int getHeight() {
		return this.height;
	}
	
	public int getCalc() {
		return this.calc;
	}
	
	public String getLightPlaneName() {
		return this.jtfLight.getText();
	}
	
	public String getDarkPlaneName() {
		return this.jtfDark.getText();
	}
	
	public String getMaskName() {
		return this.jtfMask.getText();
	}
	
	/* event listeners */
	
	// action listener methods
	public void actionPerformed(ActionEvent e) {
		String action = e.getActionCommand();
		
		if (action.equals("okay")) {
			ok();
		} else {
			cancel();
		}
	}
	
	// focus listener methods
	public void focusLost(FocusEvent e) {
		if (e.getSource() instanceof JTextField) {
			((JTextField)e.getSource()).setSelectionEnd(0);
		}
	}
	
	public void focusGained(FocusEvent e) {
		if (e.getSource() instanceof JTextField) {
			((JTextField)e.getSource()).selectAll();
		}
	}
	
	// key listener methods
	public void keyTyped(KeyEvent e) {}
	public void keyReleased(KeyEvent e) {}
	
	public void keyPressed(KeyEvent e) {
		if (e.getKeyCode() == KeyEvent.VK_ENTER) {
			ok();
		} else if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
			cancel();
		}
	}
}
