/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 */
package jspritemaker.view;

import java.util.ArrayList;

import jspritemaker.controller.CodePaneInput;
import jspritemaker.controller.ColorChooserInput;
import jspritemaker.controller.SpritePanelInput;

import jspritemaker.model.Sprite;
import jspritemaker.model.SpriteEdit;

/**
 * JSpriteMaker 68k
 * Version 1.0
 * Copyright (C) 2002 John David Ratliff
 */
public class SpriteManager {
	private ArrayList sprites;
	private SpriteView current;
	private SpritePanelInput spMouse;
	private ColorChooserInput ccMouse;
	private CodePaneInput cpInput;
	private int count;
	
	private int tool;
	
	/* constructors */
	
	public SpriteManager() {
		this.sprites = new ArrayList();
		this.current = null;
		this.count = 0;
		this.tool = SpriteEdit.POINT_TOOL;
		
		// create shared event listeners
		this.spMouse = new SpritePanelInput(this);
		this.ccMouse = new ColorChooserInput(this);
		this.cpInput = new CodePaneInput(this);
	}
	
	/* protected methods */
	
	protected void setCurrentSprite(SpriteView sprite) {
		this.current = sprite;
	}
	
	protected void setTool(int tool) {
		this.tool = tool;
	}
	
	/* public methods */
	
	public int getTool() {
		return this.tool;
	}
	
	public SpriteView getCurrentSprite() {
		return this.current;
	}
	
	public Sprite getCurrentSpriteModel() {
		if (this.current == null) {
			return null;
		}
		
		return this.current.getSprite();
	}
	
	public SpriteView createNewSprite(int width, int height, int calc, String light, String dark, String mask) {
		Sprite model = new Sprite(width,height,calc,light,dark,mask);
		SpriteView view = new SpriteView(this,model,++count);
		
		view.setSprite(model);
		view.setPanelInput(spMouse);
		view.setColorInput(ccMouse);
		view.setCodeInput(cpInput);
		this.sprites.add(view);
		this.current = view;
		
		return view;
	}
}
